/*************************************************************************
  Copyright (c) 1993 - 1997
  Digital Equipment Corporation, Maynard, Mass.

  This software is furnished under a license and may be used and  copied
  only  in  accordance  with  the  terms  of  such  license and with the
  inclusion of the above copyright notice.  This software or  any  other
  copies  thereof may not be provided or otherwise made available to any
  other person.  No title to and ownership of  the  software  is  hereby
  transferred.

  The information in this software is subject to change  without  notice
  and  should  not  be  construed  as  a commitment by DIGITAL Equipment
  Corporation.
 *************************************************************************/


/*++
* =========
* = xmt.c =
* =========
*
* Description:
*   This application demonstrates the API's ability to establish a UBR PVC
*   and transmit traffic.  This application is used in conjunction with the
*   api_rcv executable compiled from the rcv.c module.  If the entire packet
*   is sent successfully, a "." is printed.  Otherwise an "@" is printed.
*
* Arguments:
*   driver name   - The driver service name, e.g. lpa0 or lta0.
*   VPI           - The VPI to use in the PVC.
*   VCI           - The VCI to use in the PVC.
*   pkt_size      - The packet size in bytes.  Bound by 0<pkt_size<65536.
*
* Authors:
*   TLR - Theodore L. Ross
*   PAE - Paul A. Ely
*
* Modification History:
*   Date          Name     Description
*   22-Sep-1996   TLR      Created.
*   07-Jul-1997   PAE      Modified application to allow non-zero vpi values.
*   23-Jul-1997   PAE      Modified application to allow user-specifiable
*                          packet sizes to 64K bytes.
*
* Issues:
*   none.
--*/

#include <stdio.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <sys/errno.h>
#include <netaal/aal.h>

#define SIZE         65535
#define errtext(n) n

int main (int argc, char *argv[])
   {
   int                  sock;
   int                  i;
   unsigned int         value, pkt_size;
   char                 buff[SIZE];
   char                 drv_name[32];
   char                 instance;
   ATM_CONNECTION_ID    conn_id;

   /* The user must pass four command-line arguments.  The executable name,
   *    the vpi and vci of interest, and the packet size limited to 64KB.
   */

   if (argc != 5)
      {
      printf ("Usage: api_xmt drv_name vpi# vci# pkt_size\n");
      printf ("\n");
      printf ("   Where drv_name is either lpa or lta and its instance value\n");
      printf ("         vpi# is the vpi of interest\n");
      printf ("         vci# is the vci of interest\n");
      printf ("         pkt_size is the packet size in bytes, 0<pkt_size<65536\n");
      exit(1);
      }

   /* Clear out the driver name buffer. */

   for (i = 0; i < 32; i++)
      drv_name[i] = '\0';

   /* Set the local 64K buffer to a hexadecimal incrementing pattern. */

   value = 1;
   for (i = 0; i < SIZE; i++)
      {
      buff[i] = value;
      value += 1;
      if (value > 0xF) value = 1;
      }

   /* Only lpa and lta driver names are currently supported. */
 
   if (strncmp (argv[1], "lpa", 3) &&
       strncmp (argv[1], "lta", 3) != 0)
      {
      printf ("Unsupported driver service name\n");
      exit (-1);
      }

   /* Copy the driver service instance into the conn_id structure. */
   
   strncpy (&drv_name[0], argv[1], (strlen (argv[1]) + 1));

   for (i = 0; i < strlen (&drv_name[0]); i++)
      {
      if ((drv_name[i] >= '0') && (drv_name[i] <= '9'))
         {
         sscanf (&drv_name[i], "%d", &conn_id.device_number);
         break;
         }
      }

   /* Scan the user's command-line arguments and convert them into the correct
   *    type.
   */

   sscanf (argv[2], "%d", &conn_id.vpi);
   sscanf (argv[3], "%d", &conn_id.vci);
   sscanf (argv[4], "%d", &pkt_size);

   /* Open an AAL Datagram socket. */

   printf ("[ Creating ATM Socket ]\n");
   sock = socket (AF_AAL, SOCK_DGRAM, 0);   
   if (sock == -1)
      {
      printf ("[ Socket call failed, error: %d ]\n", errtext (errno));
      exit (1);
      }

   /* Notify the user of the pending association. */

   printf ("[ Associating Socket with PVC: %s/%d/%d ]\n",
	        &drv_name[0],
           conn_id.vpi,
           conn_id.vci);

   /* Create and open the PVC. */

   if (ioctl (sock, SIO_ASSOCIATE_PVC, &conn_id) == -1)
      {
      printf ("[ socket ioctl failed, error: %d ]\n", errno);
      exit (1);
      }

   /* Send the packet continuously.  When the user presses <CTRL-C>, terminate
   *    the loop and exit.
   */

   while (TRUE)
      {
      i = send (sock, buff, pkt_size, 0);
      if (i != pkt_size)
         {
         printf ("@");
         fflush (stdout);
         }
      else
         {
         printf (".");
         fflush (stdout);
         }
      }

   /* Close the socket and release all associated resources. */

   close (sock);
   return (1);
   }

